// https://www.instructables.com/Using-RP2040-PIO-in-Arduino-IDE-on-Windows/
// https://github.com/raspberrypi/pico-sdk
// https://datasheets.raspberrypi.org/pico/raspberry-pi-pico-c-sdk.pdf chapter 3.3
// https://forum.arduino.cc/t/undefined-reference-error/571942/5




extern "C" {
  #include "pio_i2c.h"
}

#define PIN_SDA 3
#define PIN_SCL 4

PIO pio = pio0;
uint sm = 0;
uint offset = pio_add_program(pio, &i2c_program);

bool reserved_addr(uint8_t addr) {
  return (addr & 0x78) == 0 || (addr & 0x78) == 0x78;
}


void setup() {
  // put your setup code here, to run once:
  pinMode(LED_BUILTIN, OUTPUT);
  Serial.begin(9600);
  i2c_program_init(pio, sm, offset, PIN_SDA, PIN_SCL);

}

void loop() {

  // put your main code here, to run repeatedly:
  static int loop;
  digitalWrite(LED_BUILTIN, HIGH);  // turn the LED on (HIGH is the voltage level)
//  Serial.println((String)"Doing this loop " + loop + " times");
  Serial.print("/");
  loop++;
  delay(1000);                      // wait for a second
  digitalWrite(LED_BUILTIN, LOW);   // turn the LED off by making the voltage LOW
  delay(1000);                      // wait for a second

  if (loop == 10) {
    Serial.print("\nPIO I2C Bus Scan\n");
    Serial.print("   0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F\n");

    for (int addr = 0; addr < (1 << 7); ++addr) {
        if (addr % 16 == 0) {
            Serial.print((String)addr + "x ");
        }
        // Perform a 0-byte read from the probe address. The read function
        // returns a negative result NAK'd any time other than the last data
        // byte. Skip over reserved addresses.
        int result;
        if (reserved_addr(addr))
            result = -1;
        else
            result = pio_i2c_read_blocking(pio, sm, addr, NULL, 0);

        Serial.print(result < 0 ? "." : "@");
        Serial.print(addr % 16 == 15 ? "\n" : "  ");
    }
    loop = 0;
  }
}
