/*
* | File        :   LCD_SPI_driver.h
* | Author      :   Leo Kuipers
* | Function    :   driver for 1.28 inch round LCD display from Waveshare
* | Based on    :   Waveshare example driver for 1.28 inch round LCD display
*/

#ifndef __LCD_DRIVER_H
#define __LCD_DRIVER_H

#include <stdint.h>
#include <stdio.h>
// #include <SPI.h>
#include <Arduino.h>
#include <WVariant.h>
#include <SERCOM.h>
#include <wiring_private.h>
#include <avr/pgmspace.h>

#define LCD_WIDTH   240 //LCD width
#define LCD_HEIGHT  240 //LCD height

#define UBYTE   uint8_t
#define UWORD   uint16_t
#define UDOUBLE uint32_t

/**
 * GPIO config
**/
//#define DEV_CS_PIN  10
#define DEV_DC_PIN  2
//#define DEV_RST_PIN 8
#define DEV_BL_PIN  4
#define MY_SERCOM sercom0

static const uint8_t MY_MISO = 30;	// sercom1 PAD[0]
static const uint8_t MY_MOSI = 14; // sercom0 PAD[0]
static const uint8_t MY_SCK = 15; // sercom0 PAD[1]
static const uint8_t MY_SS = 9;

static const EPioType MY_MOSI_PIOTYPE = PIO_SERCOM;
static const EPioType MY_MISO_PIOTYPE = PIO_SERCOM;
static const EPioType MY_SCK_PIOTYPE = PIO_SERCOM;

static const SercomSpiTXPad MY_TXPAD = SPI_PAD_0_SCK_1; // PAD[0] is output is MOSI
static const SercomRXPad MY_RXPAD = SERCOM_RX_PAD_0; 	// PAD[0] is input is MISO


/**
 * GPIO read and write
**/
#define DEV_Digital_Write(_pin, _value) digitalWrite(_pin, _value == 0? LOW:HIGH)
#define DEV_Digital_Read(_pin) digitalRead(_pin)


/**
 * SPI
**/
// #define DEV_SPI_WRITE(_dat)   SPI.transfer(_dat)
#define DEV_SPI_WRITE(_dat) MY_SERCOM.transferDataSPI(_dat);
/**
 * delay x ms
**/
#define DEV_Delay_ms(__xms)    delay(__xms)

/**
 * PWM_BL
**/
 #define  DEV_Set_BL(_Pin, _Value)  analogWrite(_Pin, _Value)

/*-----------------------------------------------------------------------------*/
 void Config_Init();
 void Start_SPI();

void LCD_WriteData_Byte(UBYTE da); 
void LCD_WriteData_Word(UWORD da);
void LCD_WriteReg(UBYTE da);

void LCD_SetCursor(UWORD x1, UWORD y1, UWORD x2,UWORD y2);
void LCD_SetUWORD(UWORD x, UWORD y, UWORD Color);

void LCD_Init(void);
void LCD_SetBacklight(UWORD Value);
void LCD_Clear(UWORD Color);
void LCD_ClearWindow(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend, UWORD UWORD);

#endif

