/*
This is a p5js sketch that uses the WebMIDI.js (webmidi.iife.js) library to receive MIDI messages

*/


var bgColor; //background color variable
var squareColor; //color of a square
var displayText; //text to be displayed
var sample; //sound to be played
var logTexts = []; // text to log to the canvas
var midiConnectButtons = []; // buttons that hold available midi in devices
var midiConnectButtonsX = 150; // location of midiConnectButtons
let synth; // synth object
let reverb, delay, distortion; // effects
let volumeSlider, reverbSlider, delaySlider, distortionSlider; // sliders to control the effects
let volumeSliderText, reverbSliderText, delaySliderText, distortionSliderText; // labels
let attackSlider, decaySlider, sustainSlider, releaseSlider;
let attackSliderText, decaySliderText, sustainSliderText, releaseSliderText;

function setup() { 
  createCanvas(600, 600);  //400 by 400 pixel canvas
  bgColor= color(220,220,200);  //starting background color
  background(bgColor);
  squareColor = color(100);  //starting square color
  displayText="";  //starting text

  // Create the polysynth
  // synth = new p5.PolySynth(); // default is 8 voices
  synth = new p5.MonoSynth(); // 1 voice only, but allows more sound manipulation like changing oscillator types
  // synth.oscillator.setType('sawtooth');

  // Create effects
  reverb = new p5.Reverb();
  delay = new p5.Delay();
  distortion = new p5.Distortion();

  // synth.disconnect();
  synth.connect(reverb);
  synth.connect(delay);
  synth.connect(distortion);

  // Create sliders
  volumeSlider = createSlider(0, 1, 0.5, 0.01);
  reverbSlider = createSlider(0, 1, 0.5, 0.01);
  delaySlider = createSlider(0, 1, 0.5, 0.01);
  distortionSlider = createSlider(0, 1, 0.5, 0.01);
  attackSlider = createSlider(0, 1, 0.01, 0.01);
  decaySlider = createSlider(0, 1, 0.01, 0.01);
  sustainSlider = createSlider(0, 1, 0.5, 0.01);
  releaseSlider = createSlider(0, 1, 0.1, 0.01);
    
  // Position sliders
  volumeSlider.position(100, 50);
  reverbSlider.position(100, 90);
  delaySlider.position(100, 130);
  distortionSlider.position(100, 170);
  attackSlider.position(100, 220);
  decaySlider.position(100, 260);
  sustainSlider.position(100, 300);
  releaseSlider.position(100, 340);

  // text labels
  volumeSliderText = createP("volume");
  volumeSliderText.position(50,35);
  reverbSliderText = createP("reverb");
  reverbSliderText.position(60,75);
  delaySliderText = createP("delay");
  delaySliderText.position(65,115);
  distortionSliderText = createP("distortion");
  distortionSliderText.position(40,155);

  attackSliderText = createP("attack");
  attackSliderText.position(60,205);
  decaySliderText = createP("decay");
  decaySliderText.position(60,245);
  sustainSliderText = createP("sustain");
  sustainSliderText.position(55,285);
  releaseSliderText = createP("release");
  releaseSliderText.position(55,325);

	WebMidi.enable(function (err) { //check if WebMidi.js is enabled
    if (err) {
        logTexts.push("WebMidi could not be enabled.", err);
      } else {
        logTexts.push("WebMidi enabled!");
      }
        
    //list all MIDI input and output ports connected to the computer
    logTexts.push("---");
    logTexts.push("Inputs Ports: ");
    text("Select MIDI input:", 10, 10);
    for(i = 0; i< WebMidi.inputs.length; i++){
      createMidiConnectButton(i + ": " + WebMidi.inputs[i].name, i);
      // createMidiConnectButton(i + ": " + "test", i);
      logTexts.push(i + ": " + WebMidi.inputs[i].name);
    }
	});
} 

function draw() { 
  background(bgColor);
  fill(0);
  textAlign(LEFT);
  textSize(15);

  text("Select MIDI input: ", 10, 15);

  updateLog();  
  fill(0);
  textAlign(CENTER);
  textSize(20);
  text(displayText,width/3,(height-75));

  // Control the synth parameters with the sliders
  synth.amp(volumeSlider.value());
  reverb.process(synth, reverbSlider.value(), 2);
  delay.process(synth, delaySlider.value(), .5, 2300);
  distortion.process(synth, distortionSlider.value());  

  let ADSRattack = attackSlider.value(); // cannot use the variablename attack because thats reserved
  let ADSRdecay = decaySlider.value();
  let ADSRsustain = sustainSlider.value();
  let ADSRrelease = releaseSlider.value();
  
  synth.setADSR(ADSRattack, ADSRdecay, ADSRsustain, ADSRrelease);  
}

function updateLog() {
  fill(0); // Set the fill color to black
  noStroke(); // No border around the text
  textSize(16); // Set the text size
  
  // Display the texts in reverse order
  for (let i = 0; i < logTexts.length; i++) {
    text(logTexts[i], 2*(width/3), height - i * 20);
  }
  
  // If the texts go beyond the canvas, remove the oldest one
  if (logTexts.length * 20 > height - 20) {
    logTexts.shift();
  }
}

function createMidiConnectButton(label, index) {
  let button = createButton(label);
  button.position(midiConnectButtonsX, 2);
  midiConnectButtonsX = midiConnectButtonsX + button.width;
  button.mousePressed(() => buttonMidiConnectClicked(index));
  button.style('background-color', 'gray');
  midiConnectButtons.push(button);
}

function buttonMidiConnectClicked(buttonNumber) {
  logTexts.push('Button ' + buttonNumber + ' clicked');
  // Change the color of all buttons to gray
  for (let i = 0; i < midiConnectButtons.length; i++) {
    midiConnectButtons[i].style('background-color', 'gray');
    if (typeof midi !== 'undefined') { // check if midi input was already used
      if (midi.hasListener('noteon')) {
        midi.removeListener(); // remove all listeners
      }
    }
  }

  midiConnectButtons[buttonNumber].style('background-color', 'green');    // Change the color of the clicked button to green
  midi = WebMidi.inputs[buttonNumber];   //Choose an input port (0 = select first midi input. Can also be a string of the portname)
  userStartAudio(); // required to tell the browser that the user did an action to start audio.
  synth.setADSR(0.1, 0.4, 0.3, 0.05); // attackTime, decayTime, susRatio, releaseTime  
  midi.addListener('noteon', "all",   //add listener function to listen to all incoming "note on" input events
    function (e) {
      let pitch = midiToFreq(e.note.number);
      let velocity = e.velocity;

      if (velocity == 0) { // note off
        logTexts.push("'noteoff' " + e.note.name + e.note.octave + " ("+ e.note.number +")");
        displayText = e.note.name + e.note.octave + "("+ e.note.number +") " + velocity * 127;
      }
      else { // note on
        //Show what we are receiving
        logTexts.push("'noteon' " + e.note.name + e.note.octave + " ("+ e.note.number +") " + velocity * 127);
        displayText = e.note.name + e.note.octave + "("+ e.note.number +") " + velocity * 127;

        //change the background color variable
        var randomR = random(0,255);
        var randomG = random(0,255);
        var randomB = random(0,255);
        bgColor = color(randomR,randomB,randomG);

        synth.play(pitch, velocity, 0, 1); // play note
      }
    }
  );
    
  //The note off functionality will need its own event listener. A lot of midi keyboards send a noteon with velocity 0 instead of noteoff. So not doing anything now.
  midi.addListener('noteoff', "all",
    function (e) {
      //Show what we are receiving
      console.log("Received 'noteoff' message (" + e.note.name + e.note.octave + ") "+ e.note.number +".");
      logTexts.push("Received 'noteoff' message (" + e.note.name + e.note.octave + ") "+ e.note.number +".");
    }
  );

}


