/*
** Firmware for Marimbatron main module
** Version UI TEST
** 
** this firmware will:
** - blink the red and green leds
** - use ISR timers to get micros
** - test Arduino Menu library 4 buttons
**
** TODO
** - implement buttons and screen for UI
** - Buttons:
** - - switch wicki-hayden <-> piano keylayout
** - - menu - (octave down)
** - - menu + (octave up)
** - - menu enter (enter to enter menu and to confirm)
**
** - store configurable values in non-volatile flash
**
**
** The work is provided as is; 
** no warranty is provided, and users accept all liability.
** 
** Copyright 2024 Leo Kuipers.
*/

/*
pin assignments based on variant.h
using one uart, one wire, one spi, USB Config: WITH_CDC:

UART
PIN_USB_DM (24ul)-> PA24
PIN_USB_DP (25ul)-> PA25

I2C (sercom 1)
PIN_WIRE_SDA (16u)-> PA16
PIN_WIRE_SCL (17u)-> PA17

SPI (sercom3)
PIN_SPI_MISO (22u)-> PA22
PIN_SPI_MOSI (18u)-> PA18
PIN_SPI_SCK (19u)-> PA19
PIN_SPI_SS (23u)-> PA23
(HW SS isn't used)

*/

#define REDLEDPIN PORT_PA00 // PA00
#define GREENLEDPIN PORT_PA01 // PA01

// timers
// CPU clock is 48 MHz (48.000.000 Hz)
// so 1 CPU clock cycle is about 20,83ns. 
// timer_hander is called every time the timer overflows.
// which is here programmed as COUNT_PER_TICK * 1_CPU_CLOCKCYCLE.
// for a 100 us timer, COUNT_PER_TICK = 10.000 ticks because
// 48.000.000 / 10.000 = 4.800 and 48.000 * 20,83ns = 99,984us which is about 100us.

#define CPU_HZ 48000000 // The cpu clock is running at 48 MHz
#define TICK_HZ_100US 10000 // 100 us tick
#define COUNTS_PER_TICK_100US (CPU_HZ / TICK_HZ_100US) // number of clock cycles per tick (4800)
#define TICK_HZ_TC5 1000000 // 10ms tick
#define COUNTS_PER_TICK_TC5 (CPU_HZ / TICK_HZ_TC5) // This is the number of clock cycles per tick (480000)

volatile uint32_t micros_counter = 0; // holds the count of microseconds since the MCU started (don't want to use micros())
const unsigned long intervalEvent1 = 10000; // in microseconds
const unsigned long intervalEvent2 = 50000; // in microseconds
const unsigned long intervalEvent3 = 500000; // in microseconds
unsigned long previousTimeEvent1 = 0;
unsigned long previousTimeEvent2 = 0;
unsigned long previousTimeEvent3 = 0;

/* ArduinoMenu example serialio */
#include <menu.h>
#include <menuIO/serialOut.h>
#include <menuIO/serialIn.h>
using namespace Menu;

result zZz() {Serial.println("zZz");return proceed;}

result showEvent(eventMask e,navNode& nav,prompt& item) {
  Serial.print("event: ");
  Serial.println(e);
  return proceed;
}

float test=55;

result action1(eventMask e) {
  Serial.print(e);
  Serial.println(" action1 executed, proceed menu");
  Serial.flush();
  return proceed;
}

result action2(eventMask e,navNode& nav,prompt& item) {
  Serial.print(e);
  Serial.print(" action2 executed, quiting menu");
  return quit;
}

int ledCtrl=LOW;

result myLedOn() {
  ledCtrl=HIGH;
  return proceed;
}
result myLedOff() {
  ledCtrl=LOW;
  return proceed;
}

TOGGLE(ledCtrl,setLed,"Led: ",doNothing,noEvent,noStyle//,doExit,enterEvent,noStyle
  ,VALUE("On",HIGH,doNothing,noEvent)
  ,VALUE("Off",LOW,doNothing,noEvent)
);

int selTest=0;
SELECT(selTest,selMenu,"Select",doNothing,noEvent,noStyle
  ,VALUE("Zero",0,doNothing,noEvent)
  ,VALUE("One",1,doNothing,noEvent)
  ,VALUE("Two",2,doNothing,noEvent)
);

int chooseTest=-1;
CHOOSE(chooseTest,chooseMenu,"Choose",doNothing,noEvent,noStyle
  ,VALUE("First",1,doNothing,noEvent)
  ,VALUE("Second",2,doNothing,noEvent)
  ,VALUE("Third",3,doNothing,noEvent)
  ,VALUE("Last",-1,doNothing,noEvent)
);

//customizing a prompt look!
//by extending the prompt class
class altPrompt:public prompt {
public:
  // altPrompt(constMEM promptShadow& p):prompt(p) {}
  using prompt::prompt;
  Used printTo(navRoot &root,bool sel,menuOut& out, idx_t idx,idx_t len,idx_t) override {
    return out.printRaw(F("special prompt!"),len);
  }
};

MENU(subMenu,"Sub-Menu",showEvent,anyEvent,noStyle
  ,OP("Sub1",showEvent,anyEvent)
  ,OP("Sub2",showEvent,anyEvent)
  ,OP("Sub3",showEvent,anyEvent)
  ,altOP(altPrompt,"",showEvent,anyEvent)
  ,EXIT("<Back")
);

uint16_t year=2017;
uint16_t month=10;
uint16_t day=7;

//define a pad style menu (single line menu)
//here with a set of fields to enter a date in YYYY/MM/DD format
//altMENU(menu,birthDate,"Birth",doNothing,noEvent,noStyle,(systemStyles)(_asPad|Menu::_menuData|Menu::_canNav|_parentDraw)
PADMENU(birthDate,"Birth",doNothing,noEvent,noStyle
  ,FIELD(year,"","/",1900,3000,20,1,doNothing,noEvent,noStyle)
  ,FIELD(month,"","/",1,12,1,0,doNothing,noEvent,wrapStyle)
  ,FIELD(day,"","",1,31,1,0,doNothing,noEvent,wrapStyle)
);

const char* constMEM hexDigit MEMMODE="0123456789ABCDEF";
const char* constMEM hexNr[] MEMMODE={"0","x",hexDigit,hexDigit};
char buf1[]="0x11";

const char* constMEM alphaNum MEMMODE=" 0123456789.ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz,\\|!\"#$%&/()=?~*^+-{}[]€";
const char* constMEM alphaNumMask[] MEMMODE={alphaNum};
char name[]="                                                  ";

MENU(mainMenu,"Main menu",zZz,noEvent,wrapStyle
  ,OP("Op1",action1,anyEvent)
  ,OP("Op2",action2,enterEvent)
  ,FIELD(test,"Test","%",0,100,10,1,doNothing,noEvent,wrapStyle)
  ,SUBMENU(subMenu)
  ,SUBMENU(setLed)
  ,OP("LED On",myLedOn,enterEvent)
  ,OP("LED Off",myLedOff,enterEvent)
  ,SUBMENU(selMenu)
  ,SUBMENU(chooseMenu)
  ,OP("Alert test",doAlert,enterEvent)
  ,EDIT("Name",name,alphaNumMask,doNothing,noEvent,noStyle)
  ,EDIT("Hex",buf1,hexNr,doNothing,noEvent,noStyle)
  ,SUBMENU(birthDate)
  ,EXIT("<Back")
);

#define MAX_DEPTH 2

MENU_OUTPUTS(out,MAX_DEPTH
  ,SERIAL_OUT(Serial)
  ,NONE//must have 2 items at least
);


serialIn serial(Serial);

NAVROOT(nav,mainMenu,MAX_DEPTH,serial,out);

result alert(menuOut& o,idleEvent e) {
  if (e==idling) {
    o.setCursor(0,0);
    o.print("alert test");
    o.setCursor(0,1);
    o.print("press [select]");
    o.setCursor(0,2);
    o.print("to continue...");
  }
  return proceed;
}

result doAlert(eventMask e, prompt &item) {
  nav.idleOn(alert);
  return proceed;
}

result idle(menuOut &o, idleEvent e) {
  // o.clear();
  switch(e) {
    // case idleStart:o.println("suspending menu!");break;
    case idling:o.println("menu suspended... Hit enter to wake up.");break;
    // case idleEnd:o.println("resuming menu.");
      nav.reset();
      break;
  }
  return proceed;
}
/* end arduino menu example */


void TC4_Handler() { // 100us timer, priority 0
  micros_counter = micros_counter + 100; // add 100 us to the micros_counter
  TC4->COUNT16.INTFLAG.bit.MC0 = 1;  // Clear the interrupt
}

void TC5_Handler() { // sensor timer, priority 1
  TC5->COUNT16.INTFLAG.bit.MC0 = 1;  // Clear the interrupt
}

void setup() {
  // set SAMD to run at 48Mhz
  setup48MhzUSBClockRecoveryMode();
  setupHardwareTimers();  

  Serial.begin(115200);
  while (!Serial && micros_counter < 5000000); // wait for serial bus to activate (in order to communicate immediately after reset) or timeout
  delay(100);
  Serial.print("Marimbatron Main board firmware TEST UI");

  PORT->Group[0].DIRSET.reg = REDLEDPIN; // set led pin as output
  PORT->Group[0].DIRSET.reg = GREENLEDPIN; // set led pin as output
  PORT->Group[0].OUTSET.reg = REDLEDPIN; // turn RED on
  PORT->Group[0].OUTCLR.reg = GREENLEDPIN; // turn GREEN off

  // set PA10 to input, enabling pull resistor and setting it to pull-up.
  PORT->Group[0].DIRCLR.reg = PORT_PA10;
  PORT->Group[0].PINCFG[10].reg = PORT_PINCFG_PULLEN | PORT_PINCFG_INEN;
  PORT->Group[0].OUTSET.reg = PORT_PA10;
  // set PA11 to input, enabling pull resistor and setting it to pull-up.
  PORT->Group[0].DIRCLR.reg = PORT_PA11;
  PORT->Group[0].PINCFG[11].reg = PORT_PINCFG_PULLEN | PORT_PINCFG_INEN;
  PORT->Group[0].OUTSET.reg = PORT_PA11;
  // set PA14 to input, enabling pull resistor and setting it to pull-up.
  PORT->Group[0].DIRCLR.reg = PORT_PA14;
  PORT->Group[0].PINCFG[14].reg = PORT_PINCFG_PULLEN | PORT_PINCFG_INEN;
  PORT->Group[0].OUTSET.reg = PORT_PA14;
  // set PA15 to input, enabling pull resistor and setting it to pull-up.
  PORT->Group[0].DIRCLR.reg = PORT_PA15;
  PORT->Group[0].PINCFG[15].reg = PORT_PINCFG_PULLEN | PORT_PINCFG_INEN;
  PORT->Group[0].OUTSET.reg = PORT_PA15;

 /* Arduinomenu example serialio */
  if (ledCtrl){
    PORT->Group[0].OUTSET.reg = REDLEDPIN;
  }
  else {
    PORT->Group[0].OUTCLR.reg = REDLEDPIN;
  }
  delay(500);
  Serial.println("menu 4.x test");Serial.flush();
  nav.timeOut=5;
  nav.idleTask=idle; //point a function to be used when menu is suspended
  // nav.idleOn();//this menu will start on idle state, press select to enter menu
  //nav.doInput("323");
  // nav.useAccel=false;

  /* end Arduino menu example */

  PORT->Group[0].OUTCLR.reg = REDLEDPIN; // turn RED off
  PORT->Group[0].OUTCLR.reg = GREENLEDPIN; // turn GREEN off
  Serial.println("Entering loop()");  
}

void loop() {
  static bool btn0State = 0;
  static bool btn1State = 0;
  static bool btn2State = 0;
  static bool btn3State = 0;
  static bool btn0StateOld = 0;
  static bool btn1StateOld = 0;
  static bool btn2StateOld = 0;
  static bool btn3StateOld = 0;
  /* Arduino menu example serial io */
  nav.poll(); // deals with all I/O, driving the menu on a full automated mode.
  if (ledCtrl){
    PORT->Group[0].OUTSET.reg = REDLEDPIN;
  }
  else {
    PORT->Group[0].OUTCLR.reg = REDLEDPIN;
  }
/* end arduinomenu example */

  unsigned long currentTime = micros_counter;
  /* This is event 1 -> request sensor data */
  if ((unsigned long)(currentTime - previousTimeEvent1) >= intervalEvent1) {
    btn0State = ~PORT->Group[0].IN.reg & PORT_PA10 ? 1 : 0; // read value of input pin. Input has pull-up so the actual value is inverted. this converts back.
    btn1State = ~PORT->Group[0].IN.reg & PORT_PA14 ? 1 : 0; // read value of input pin. Input has pull-up so the actual value is inverted. this converts back.
    btn2State = ~PORT->Group[0].IN.reg & PORT_PA11 ? 1 : 0; // read value of input pin. Input has pull-up so the actual value is inverted. this converts back.
    btn3State = ~PORT->Group[0].IN.reg & PORT_PA15 ? 1 : 0; // read value of input pin. Input has pull-up so the actual value is inverted. this converts back.

   /* Update the timing for the next time around */
    previousTimeEvent1 = currentTime;
  }

  /* This is event 2 -> check for and send noteoff */
  if ((unsigned long)(currentTime - previousTimeEvent2) >= intervalEvent2) {
    if (btn0State != btn0StateOld) {
      // Serial.print("Button 0: ");Serial.println(btn0State);
      if (btn0State) {
        nav.doNav(escCmd);
      }
      btn0StateOld = btn0State;
    }
    if (btn1State != btn1StateOld) {
      // Serial.print("Button 1: ");Serial.println(btn1State);
      if (btn1State) {
        nav.doNav(upCmd);
      }
      btn1StateOld = btn1State;
    }
    if (btn2State != btn2StateOld) {
      // Serial.print("Button 2: ");Serial.println(btn2State);
      if (btn2State) {
        nav.doNav(downCmd);
      }
      btn2StateOld = btn2State;
    }
    if (btn3State != btn3StateOld) {
      // Serial.print("Button 3: ");Serial.println(btn3State);
      if (btn3State) {
        nav.doNav(enterCmd);
      }
      btn3StateOld = btn3State;
    }

   /* Update the timing for the next time around */
    previousTimeEvent2 = currentTime;
  }  
  /* This is event 3 -> blink Green LED */
  if ((unsigned long)(currentTime - previousTimeEvent3) >= intervalEvent3) {
    PORT->Group[0].OUTTGL.reg = GREENLEDPIN; // toggle green led pin as indicator that the main controller is still operating properly

   /* Update the timing for the next time around */
    previousTimeEvent3 = currentTime;
  }
}


void setup48MhzUSBClockRecoveryMode(){
  // from https://blog.thea.codes/understanding-the-sam-d21-clocks/
  /* Set the correct number of wait states for 48 MHz @ 3.3v */
  NVMCTRL->CTRLB.bit.RWS = 1;

  /* This works around a quirk in the hardware (errata 1.2.1) -
    the DFLLCTRL register must be manually reset to this value before
    configuration. */
  while(!SYSCTRL->PCLKSR.bit.DFLLRDY);
  SYSCTRL->DFLLCTRL.reg = SYSCTRL_DFLLCTRL_ENABLE;
  while(!SYSCTRL->PCLKSR.bit.DFLLRDY);

  /* Write the coarse and fine calibration from NVM. */
  uint32_t coarse =
      ((*(uint32_t*)FUSES_DFLL48M_COARSE_CAL_ADDR) & FUSES_DFLL48M_COARSE_CAL_Msk) >> FUSES_DFLL48M_COARSE_CAL_Pos;
  uint32_t fine =
      ((*(uint32_t*)FUSES_DFLL48M_FINE_CAL_ADDR) & FUSES_DFLL48M_FINE_CAL_Msk) >> FUSES_DFLL48M_FINE_CAL_Pos;

  SYSCTRL->DFLLVAL.reg = SYSCTRL_DFLLVAL_COARSE(coarse) | SYSCTRL_DFLLVAL_FINE(fine);

  /* Wait for the write to finish. */
  while (!SYSCTRL->PCLKSR.bit.DFLLRDY) {};

  SYSCTRL->DFLLCTRL.reg |=
      /* Enable USB clock recovery mode */
      SYSCTRL_DFLLCTRL_USBCRM |
      /* Disable chill cycle as per datasheet to speed up locking.
        This is specified in section 17.6.7.2.2, and chill cycles
        are described in section 17.6.7.2.1. */
      SYSCTRL_DFLLCTRL_CCDIS;

  /* Configure the DFLL to multiply the 1 kHz clock to 48 MHz */
  SYSCTRL->DFLLMUL.reg =
      /* This value is output frequency / reference clock frequency,
        so 48 MHz / 1 kHz */
      SYSCTRL_DFLLMUL_MUL(48000) |
      /* The coarse and fine values can be set to their minimum
        since coarse is fixed in USB clock recovery mode and
        fine should lock on quickly. */
      SYSCTRL_DFLLMUL_FSTEP(1) |
      SYSCTRL_DFLLMUL_CSTEP(1);

  /* Closed loop mode */
  SYSCTRL->DFLLCTRL.bit.MODE = 1;

  /* Enable the DFLL */
  SYSCTRL->DFLLCTRL.bit.ENABLE = 1;

  /* Wait for the write to complete */
  while (!SYSCTRL->PCLKSR.bit.DFLLRDY) {};      

  /* Setup GCLK0 using the DFLL @ 48 MHz */
  GCLK->GENCTRL.reg =
      GCLK_GENCTRL_ID(0) |
      GCLK_GENCTRL_SRC_DFLL48M |
      /* Improve the duty cycle. */
      GCLK_GENCTRL_IDC |
      GCLK_GENCTRL_GENEN;

  /* Wait for the write to complete */
  while(GCLK->STATUS.bit.SYNCBUSY) {};

  // for debugging: temporarily set 48MHz clock to GCLK4 as well and output to a pin to check accuracy.
  // GCLK->GENCTRL.reg =
  //     GCLK_GENCTRL_ID(4) |
  //     GCLK_GENCTRL_SRC_DFLL48M |
  //     GCLK_GENCTRL_IDC |
  //     GCLK_GENCTRL_GENEN |
  //     /* Enable outputting the clock to an I/O pin. */
  //     GCLK_GENCTRL_OE;

  // /* Wait for the write to complete */
  // while(GCLK->STATUS.bit.SYNCBUSY);
  // PORT->Group[1].DIRSET.reg = (1 << 10);
  // PORT->Group[1].PINCFG[10].reg |= PORT_PINCFG_PMUXEN;
  // PORT->Group[1].PMUX[10 >> 1].bit.PMUXE |= PORT_PMUX_PMUXE_H;
}

void setupHardwareTimers() 
{
    // Set up the generic clock (GCLK4) used to clock timers
    GCLK->GENDIV.reg = GCLK_GENDIV_DIV(1) |          // Divide the 48MHz clock source by divisor 1: 48MHz/1=48MHz
                       GCLK_GENDIV_ID(4);            // Select Generic Clock (GCLK) 4
    while (GCLK->STATUS.bit.SYNCBUSY);               // Wait for synchronization

    GCLK->GENCTRL.reg = GCLK_GENCTRL_IDC |           // Set the duty cycle to 50/50 HIGH/LOW
                        GCLK_GENCTRL_GENEN |         // Enable GCLK4
                        GCLK_GENCTRL_SRC_DFLL48M |   // Set the 48MHz clock source
                        GCLK_GENCTRL_ID(4);          // Select GCLK4
    while (GCLK->STATUS.bit.SYNCBUSY);               // Wait for synchronization

    // // Feed GCLK4 to TC3
    // GCLK->CLKCTRL.reg = GCLK_CLKCTRL_CLKEN |         // Enable GCLK4
    //                     GCLK_CLKCTRL_GEN_GCLK4 |     // Select GCLK4
    //                     GCLK_CLKCTRL_ID_TCC2_TC3;     // Feed the GCLK4 to TC3 (and TCC2)
    // while (GCLK->STATUS.bit.SYNCBUSY);               // Wait for synchronization

    // Feed GCLK4 to TC4 and TC5
    GCLK->CLKCTRL.reg = GCLK_CLKCTRL_CLKEN |         // Enable GCLK4
                        GCLK_CLKCTRL_GEN_GCLK4 |     // Select GCLK4
                        GCLK_CLKCTRL_ID_TC4_TC5;     // Feed the GCLK4 to TC4 and TC5
    while (GCLK->STATUS.bit.SYNCBUSY);               // Wait for synchronization

    // // specific for TC3: 1ms
    // TC3->COUNT16.CTRLA.reg |= TC_CTRLA_MODE_COUNT16;  // Set Timer counter Mode to 16 bits
    // while (TC3->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // TC3->COUNT16.CTRLA.reg |= TC_CTRLA_WAVEGEN_MFRQ;  // Set TC as Match Frequency
    // while (TC3->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // TC3->COUNT16.CC[0].reg = COUNTS_PER_TICK_1MS;     // Set the TC3 CC0 register as the TOP value in match frequency mode
    // while (TC3->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // // Enable the TC3 interrupt request
    // TC3->COUNT16.INTENSET.bit.MC0 = 1;
    // while (TC3->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // // Enable TC3
    // TC3->COUNT16.CTRLA.reg |= TC_CTRLA_ENABLE;        // Enable TC3
    // while (TC3->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // // Enable the TC3 interrupt vector use priority 2
    // NVIC_SetPriority(TC3_IRQn, 2);                    // Set the NVIC priority for TC3 to 2
    // NVIC_EnableIRQ(TC3_IRQn);

    // specific for TC4: 100us
    TC4->COUNT16.CTRLA.reg |= TC_CTRLA_MODE_COUNT16;  // Set Timer counter Mode to 16 bits
    while (TC4->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    TC4->COUNT16.CTRLA.reg |= TC_CTRLA_WAVEGEN_MFRQ;  // Set TC as Match Frequency
    while (TC4->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    TC4->COUNT16.CC[0].reg = COUNTS_PER_TICK_100US;     // Set the TC4 CC0 register as the TOP value in match frequency mode
    while (TC4->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable the TC4 interrupt request
    TC4->COUNT16.INTENSET.bit.MC0 = 1;
    while (TC4->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable TC4
    TC4->COUNT16.CTRLA.reg |= TC_CTRLA_ENABLE;        // Enable TC4
    while (TC4->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable the TC4 interrupt vector use priority 0
    NVIC_SetPriority(TC4_IRQn, 0);                    // Set the NVIC priority for TC4 to 0
    NVIC_EnableIRQ(TC4_IRQn);

    // specific for TC5
    TC5->COUNT16.CTRLA.reg |= TC_CTRLA_MODE_COUNT16;  // Set Timer counter Mode to 16 bits
    while (TC5->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    TC5->COUNT16.CTRLA.reg |= TC_CTRLA_WAVEGEN_MFRQ;  // Set TC as Match Frequency
    while (TC5->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    TC5->COUNT16.CC[0].reg = COUNTS_PER_TICK_TC5;     // Set the TC5 CC0 register as the TOP value in match frequency mode
    while (TC5->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable the TC5 interrupt request
    TC5->COUNT16.INTENSET.bit.MC0 = 1;
    while (TC5->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable TC5
    TC5->COUNT16.CTRLA.reg |= TC_CTRLA_ENABLE;        // Enable TC5
    while (TC5->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // use priority 1
    NVIC_SetPriority(TC5_IRQn, 1);                    // Set the NVIC priority for TC5 to 1
    NVIC_EnableIRQ(TC5_IRQn);    // Enable the TC5 timer (start checking sensorboards for hits)

}
