/*
** Firmware for Marimbatron Sensormodule
** Version peripheralReadTest

**
** Required libraries:
** wire.h
**
** This work may be reproduced, modified, distributed, performed, and 
** displayed for any non-commercial, non-profit purpose, but must acknowledge 
** the developer and the Marimbatron project. 
** Copyright is retained and must be preserved. The work is 
** provided as is; no warranty is provided, and users accept all liability.
** 
** Copyright 2024 Leo Kuipers.
*/

#define FIRMWAREVERSION 0

// LEDs and WS2812B
#define GREENLEDPIN PORT_PB30 // PB30 = arduino 43
#define REDLEDPIN PORT_PB31 // PB31 = arduino 46

// I2C communication
#include <Wire.h> 
static uint8_t mode = 0; // program mode: S = sensor mode, R = reset this controller

void I2CCommand(int length)
{
  delay(1);
  Serial.print("Receiving command with length "); Serial.println(length);
  if (length > 0) { // if there's no length, there are no command bytes. So it's a request for data which is handled by I2CEvent()
    Serial.print("Current mode is "); Serial.println(mode);
    for (uint8_t l = 0; l < length; l++) { // read I2C data
      Serial.print("Reading I2C ");Serial.print(l);Serial.print(" of ");Serial.println(length);
      uint8_t c = Wire1.read(); // read first byte
      if (c == 255) { // read error, no data available
        Serial.println("Error reading I2C data");
        // PORT->Group[1].OUTTGL.reg = REDLEDPIN; // turn green led pin on
      }
      else {
        mode = c; 
        Serial.print("mode received. Mode is now ");
        Serial.println(mode);
        PORT->Group[1].OUTTGL.reg = GREENLEDPIN; // turn green led pin on
      }
    }
    if (mode == 1) {
      Serial.println("Reset received.");
      // NVIC_SystemReset(); // reset this microcontroller
    }
  }
}

void I2CEvent()
{
  uint16_t velocity;
  bool hitDetected = false;
  uint8_t data[] = {255,0,0};
  Serial.println("Received I2C event");
  if (mode == 0) {
    // send [255, 0, 0] when no hit is detected. Otherwise send sensor number and velocity of just 1 sensor and reset the hit state of that sensor
    data[0] = 1;
    data[1] = 1;
    data[2] = 1;
  }
    Serial.print("sending I2C data: ");
    Serial.print(data[0]);Serial.print(" ");Serial.print(data[1]);Serial.print(" ");Serial.println(data[2]);
    Wire1.write(data, 3); // even when no hit is detected an answer will still be send using sensor number 255 (255, 0, 0)
}

void setup() 
{
  // configure LED pins
  PORT->Group[1].DIRSET.reg = GREENLEDPIN | REDLEDPIN; // set led pins as output
  PORT->Group[1].OUTSET.reg = REDLEDPIN; // turn red led pin on
  PORT->Group[1].OUTCLR.reg = GREENLEDPIN; // turn green led pin off

  Wire1.begin(1); // start I2C with address based on jumpers

  // configure serial port
  Serial.begin(115200); // enable serial communication  
  while (!Serial); // wait for serial bus to activate (in order to communicate immediately after reset) or timeout
  delay(100);
  Serial.print("Marimbatron Sensor board firmware test v");
  Serial.println(FIRMWAREVERSION);

  // Wire1.onRequest(I2CEvent); // start listening for I2C requests from main where it is requested to send data back
  Wire1.onReceive(I2CCommand); // start listening for I2C commands send by main to this peripheral  
  PORT->Group[1].OUTSET.reg = GREENLEDPIN; // turn green led pin on
  PORT->Group[1].OUTCLR.reg = REDLEDPIN; // turn red led pin off

}

void loop() 
{
  // Check the I2C overrun flag
  if (SERCOM2->I2CM.STATUS.bit.BUSERR) {
    PORT->Group[1].OUTTGL.reg = REDLEDPIN; // turn green led pin on
  }
}
