/*
** Firmware for Marimbatron Sensormodule
** Sensor.h
** 
** this file defines the sensor class
**
** This work may be reproduced, modified, distributed, performed, and 
** displayed for any non-commercial, non-profit purpose, but must acknowledge 
** the developer and the Marimbatron project. 
** Copyright is retained and must be preserved. The work is 
** provided as is; no warranty is provided, and users accept all liability.
** 
** Copyright 2024 Leo Kuipers.
*/

#ifndef Sensor_h // include guard preventing this file to be included twice
#define Sensor_h
#include "Arduino.h" // make arduino functions available to this class
class Sensor {
    public: // Public functions and variables can be used inside the sketch
        Sensor(/* args */); // constructor
        void setThreshold(uint32_t newThreshold);
        void setScanTime(uint32_t newScantime);
        void setMaskTime(uint32_t newMasktime);
        void setLastADCTime(unsigned long newLastADCTime);
        void setVelocity(uint32_t newVelocity);
        void setHit(bool newHit);
        uint32_t getThreshold();
        uint32_t getScanTime();
        uint32_t getMaskTime();
        unsigned long getLastADCTime();
        uint32_t getVelocity();
        bool getHit();

    private: // private functions and variables can only be used inside the class.
    // naming convention for private variables is with an underscore 
        uint32_t _threshold; // threshold value above which the ADC start to detect local maximum
        uint32_t _scanTime; // amount of time the ADC should measure to get the local maximum (after threshold is passed)
        uint32_t _maskTime; // amount of time after a hit, that the ADC should be bypassed, to prevent false detections 
        unsigned long _lastADCTime; // micros() of the last time this sensor was measured (required to deal with maskTime)
        uint32_t _velocity; // local maximum, i.e. the velocity of the hit
        bool _hit; // boolean to indicate if a hit was registered. When true, the velocity variable holds a valid velocity

};

#endif // end include guard