/*
** Firmware for Marimbatron Sensormodule
** Version 2
** 
** this firmware will:
** - blink the red and green leds
** - drive 1 WS2812B RGB led and cycle the colorwheel
** - read out 19 analog sensor inputs and output their value in the serial monitor
** - use a ISR timer to read these sensor inputs
**
** todo: see if I can use DMA WS2812B library like https://github.com/adafruit/Adafruit_NeoPixel_ZeroDMA
** because neopixel library diasbles interrupts and that can mess up timing.
**
** Required libraries:
** SAMD_TimerInterrupt https://github.com/khoih-prog/SAMD_TimerInterrupt 
** Adafruit_NeoPixel https://github.com/adafruit/Adafruit_NeoPixel
**
** This work may be reproduced, modified, distributed, performed, and 
** displayed for any non-commercial, non-profit purpose, but must acknowledge 
** the developer and the Marimbatron project. 
** Copyright is retained and must be preserved. The work is 
** provided as is; no warranty is provided, and users accept all liability.
** 
** Copyright 2024 Leo Kuipers.
*/

/*
  conversion table for pin vs input vs mux vs Marimbatron sensor inputs
    MUXPOS_PIN0  = AIN0  = PA02 -> SENSOR4
    MUXPOS_PIN1  = AIN1  = PA03 -> SENSOR5
    MUXPOS_PIN2  = AIN2  = PB08 -> SENSOR19
    MUXPOS_PIN3  = AIN3  = PB09 -> SENSOR14
    MUXPOS_PIN4  = AIN4  = PA04 -> SENSOR18
    MUXPOS_PIN5  = AIN5  = PA05 -> SENSOR13
    MUXPOS_PIN6  = AIN6  = PA06 -> SENSOR17
    MUXPOS_PIN7  = AIN7  = PA07 -> SENSOR12
    MUXPOS_PIN8  = AIN8  = PB00 -> SENSOR8
    MUXPOS_PIN9  = AIN9  = PB01 -> SENSOR9
    MUXPOS_PIN10 = AIN10 = PB02 -> SENSOR6
    MUXPOS_PIN11 = AIN11 = PB03 -> SENSOR7
    MUXPOS_PIN12 = AIN12 = PB04 -> SENSOR2
    MUXPOS_PIN13 = AIN13 = PB05 -> SENSOR3
    MUXPOS_PIN14 = AIN14 = PB06 -> SENSOR0
    MUXPOS_PIN15 = AIN15 = PB07 -> SENSOR1
    MUXPOS_PIN16 = AIN16 = PA08 -> SENSOR16
    MUXPOS_PIN17 = AIN17 = PA09 -> SENSOR11
    MUXPOS_PIN18 = AIN18 = PA10 -> SENSOR15
    MUXPOS_PIN19 = AIN19 = PA11 -> SENSOR10
*/

// ISR timer and loop() event timers
#define TIMER_INTERRUPT_DEBUG         0 // This define must be placed at the beginning before #include "SAMDTimerInterrupt.h"
#define _TIMERINTERRUPT_LOGLEVEL_     1 // This define must be placed at the beginning before #include "SAMDTimerInterrupt.h"
#define USING_TIMER_TC3               true
#include "SAMDTimerInterrupt.h"
#define TIMER_INTERVAL_US             1000
#define SELECTED_TIMER                TIMER_TC3
SAMDTimer ITimer(SELECTED_TIMER); // Init selected SAMD timer
const unsigned long intervalEvent1 = 500;
const unsigned long intervalEvent2 = 1000;
unsigned long previousTimeEvent1 = 0;
unsigned long previousTimeEvent2 = 0;

// WS2812B library and LEDs
//#include <Adafruit_NeoPixel.h>
// LED output pins, including WS2818B data output
#define GREENLEDPIN 43 // PB30 = arduino 43
#define REDLEDPIN 46 // PB31 = arduino 46
#define RGBLEDPIN 39 // PB10 = arduino 39
#define NUMPIXELS 1
//Adafruit_NeoPixel pixels(NUMPIXELS, RGBLEDPIN, NEO_GRB + NEO_KHZ800);
int colorvalue = 0;
bool ledStatus;

// ADC
// mapping muxpos pins to Marimbatron sensor inputs
#define SENSOR4  ADC_INPUTCTRL_MUXPOS_PIN0
#define SENSOR5  ADC_INPUTCTRL_MUXPOS_PIN1
#define SENSOR19 ADC_INPUTCTRL_MUXPOS_PIN2
#define SENSOR14 ADC_INPUTCTRL_MUXPOS_PIN3
#define SENSOR18 ADC_INPUTCTRL_MUXPOS_PIN4
#define SENSOR13 ADC_INPUTCTRL_MUXPOS_PIN5
#define SENSOR17 ADC_INPUTCTRL_MUXPOS_PIN6
#define SENSOR12 ADC_INPUTCTRL_MUXPOS_PIN7
#define SENSOR8  ADC_INPUTCTRL_MUXPOS_PIN8
#define SENSOR9  ADC_INPUTCTRL_MUXPOS_PIN9
#define SENSOR6  ADC_INPUTCTRL_MUXPOS_PIN10
#define SENSOR7  ADC_INPUTCTRL_MUXPOS_PIN11
#define SENSOR2  ADC_INPUTCTRL_MUXPOS_PIN12
#define SENSOR3  ADC_INPUTCTRL_MUXPOS_PIN13
#define SENSOR0  ADC_INPUTCTRL_MUXPOS_PIN14
#define SENSOR1  ADC_INPUTCTRL_MUXPOS_PIN15
#define SENSOR16 ADC_INPUTCTRL_MUXPOS_PIN16
#define SENSOR11 ADC_INPUTCTRL_MUXPOS_PIN17
#define SENSOR15 ADC_INPUTCTRL_MUXPOS_PIN18
#define SENSOR10 ADC_INPUTCTRL_MUXPOS_PIN19

// array of the sensors, to be used for iterating
const int sensorArrayLength = 19;
const unsigned long sensorArray[sensorArrayLength] = {
  SENSOR0, SENSOR1, SENSOR2, SENSOR3, SENSOR4,
  SENSOR5, SENSOR6, SENSOR7, SENSOR8, SENSOR9,
  SENSOR10, SENSOR11, SENSOR12, SENSOR13, SENSOR14,
  SENSOR15, SENSOR16, SENSOR17, SENSOR18
};

#include "Sensor.h" // include sensor class
Sensor pads[sensorArrayLength]; // array that holds all sensor related data
const uint32_t DEFAULTTHRESHOLD = 3000; // 12-bit threshold, max value 4095.
const uint32_t DEFAULTSCANTIME = 2000; // scantime in us.
const uint32_t DEFAULTMASKTIME = 8000; // masktime in us.

//////////////////////////////////////////////
////////////// TIMER ISR /////////////////////
//////////////////////////////////////////////

void TimerHandler() {
  uint32_t padThreshold;
  unsigned long startScan;
  uint32_t currentMeasurement;
  // Serial.println("timer IRQ");
  for (int i = 0; i < sensorArrayLength; i++) {
    startScan = micros();
    // Serial.println((String)"startScan=" + startScan);
    // true if the measurement should not be masked. i.e. the measurement should be done.
    // startscan - lastADCTime >= masktime is done in this order to prevent issues from micros() overflow.
    // note (unsigned long) is typecasting the calculation between () to make sure the overflow prevention works.
    // see https://www.norwegiancreations.com/2018/10/arduino-tutorial-avoiding-the-overflow-issue-when-using-millis-and-micros/
    if ((unsigned long)(startScan - pads[i].getLastADCTime()) >= pads[i].getMaskTime()) {
      // Serial.println((String)"start measuring pad " + i);
      padThreshold = pads[i].getThreshold();
      setADCToPin(sensorArray[i]);
      currentMeasurement = readADC(true); // true = first measurement should be disregarded due to channel change
      // Serial.println((String)"ADC read value " + currentMeasurement + ". Threshold = " + padThreshold);
      if (currentMeasurement < padThreshold) { // measurement below threshold = hit detected. Start peak detection for scantime us.
        // Serial.println("hit detected!");
        ITimer.detachInterrupt(); // disable interrupt
        uint32_t scanTime = pads[i].getScanTime();
        // Serial.println((String)"Scantime = " + scanTime);
        uint32_t peakMeasurement = padThreshold;
        while ((unsigned long)(micros() - startScan) <= scanTime) { // again using micros() overflow prevention
          // Serial.print(".");
          currentMeasurement = readADC(false); // disregarding first measurement is not neccesary because we didn't change the input channel
          if (currentMeasurement < peakMeasurement) { // peak detection for local minimum
            peakMeasurement = currentMeasurement;
          }
        }
        // Serial.println((String)" peak: " + peakMeasurement);
        pads[i].setVelocity(peakMeasurement); // peak detection done. Store peak value.
        pads[i].setHit(true); // set hit to true to indicate the pad was hit.
        pads[i].setLastADCTime(startScan + scanTime); // store lastADCTime, required for masktime calculation.
        // Serial.println("Enabling interrupt"); 
        ITimer.reattachInterrupt(); // enable interrupt
      } 
    }
  }
}

//////////////////////////////////////////////

void setup() {
  // set SAMD21 to run at 48Mhz
  setup48MhzUSBClockRecoveryMode();

  // configure LED pins
  pinMode(GREENLEDPIN, OUTPUT); // set LED pin to output
  pinMode(REDLEDPIN, OUTPUT); // set LED pin to output
  digitalWrite(GREENLEDPIN, HIGH); // turn LED on
  digitalWrite(REDLEDPIN, LOW); // turn LED off

  // configure WS2812B
  // pixels.begin();
  // pixels.clear();
  // pixels.setBrightness(50); // Set BRIGHTNESS to about 1/5 (max = 255)


  // configure analog inputs
  setupADC();

  // configure serial port
  Serial.begin(115200); // enable serial communication
  while (!Serial && millis() < 5000); // wait for serial bus to activate (in order to communicate immediately after reset) or timeout
  delay(100);
  Serial.print(F("\nMarimbatron Sensor board firmware v2 on ")); Serial.println(BOARD_NAME);
  Serial.print(F("CPU Frequency = ")); Serial.print(F_CPU / 1000000); Serial.println(F(" MHz"));

  // configure pads
  setupPads();

  // activate timer ISR
  setupISRTimer();

  digitalWrite(GREENLEDPIN, LOW); // turn LED off
}

void loop() {
  unsigned long currentTime = millis();
  uint32_t ADCMeasurement;

  /* This is event 1 */
  if ((unsigned long)(currentTime - previousTimeEvent1) >= intervalEvent1) {
    /* Event code */
    digitalWrite(REDLEDPIN, ledStatus);
    ledStatus = !ledStatus;
    // for (int i = 0; i < sensorArrayLength; i++) {
    //   Serial.print((String)"p" + i + "=" + pads[i].getHit() + " ");
    // }
    // Serial.println();

    for (int i = 0; i < sensorArrayLength; i++) {
      if (pads[i].getHit()){ // if true: hit is detected
        Serial.println((String)"HIT on pad " + i + " with velocity " + pads[i].getVelocity());
        pads[i].setHit(false); // reset hit
      }
    }
    
   /* Update the timing for the next time around */
    previousTimeEvent1 = currentTime;
  }

  /* This is event 2*/
  if ((unsigned long)(currentTime - previousTimeEvent2) >= intervalEvent2) {
    /* Event code */
    // int pixelColor = map(colorvalue, 900, 1023, 0, 65535); // map(value, fromLow, fromHigh, toLow, toHigh)
    // int color = pixels.gamma32(pixels.ColorHSV(pixelColor)); // hue -> RGB
    // pixels.setPixelColor(0, color); // Set pixel 'c' to value 'color'
    // pixels.show(); // Update strip with new contents
    // colorvalue = colorvalue + 100;
   /* Update the timing for the next time around */
    previousTimeEvent2 = currentTime;
  }
}

void setupPads(){
  for (int i = 0; i <= sensorArrayLength; i++) {
    pads[i].setThreshold(500); // setting very low now because no sensor is attached.
    pads[i].setScanTime(DEFAULTSCANTIME);
    pads[i].setMaskTime(DEFAULTMASKTIME);
  }
  pads[8].setThreshold(DEFAULTTHRESHOLD);

}

void setupISRTimer(){
  if (ITimer.attachInterruptInterval(TIMER_INTERVAL_US, TimerHandler)) {
    Serial.print(F("Starting  ITimer OK"));
  }
  else {
    Serial.println(F("Can't set ITimer. Select another freq. or timer"));
  }
}

void setup48MhzUSBClockRecoveryMode(){
  // from https://blog.thea.codes/understanding-the-sam-d21-clocks/
  /* Set the correct number of wait states for 48 MHz @ 3.3v */
  NVMCTRL->CTRLB.bit.RWS = 1;

  /* This works around a quirk in the hardware (errata 1.2.1) -
    the DFLLCTRL register must be manually reset to this value before
    configuration. */
  while(!SYSCTRL->PCLKSR.bit.DFLLRDY);
  SYSCTRL->DFLLCTRL.reg = SYSCTRL_DFLLCTRL_ENABLE;
  while(!SYSCTRL->PCLKSR.bit.DFLLRDY);

  /* Write the coarse and fine calibration from NVM. */
  uint32_t coarse =
      ((*(uint32_t*)FUSES_DFLL48M_COARSE_CAL_ADDR) & FUSES_DFLL48M_COARSE_CAL_Msk) >> FUSES_DFLL48M_COARSE_CAL_Pos;
  uint32_t fine =
      ((*(uint32_t*)FUSES_DFLL48M_FINE_CAL_ADDR) & FUSES_DFLL48M_FINE_CAL_Msk) >> FUSES_DFLL48M_FINE_CAL_Pos;

  SYSCTRL->DFLLVAL.reg = SYSCTRL_DFLLVAL_COARSE(coarse) | SYSCTRL_DFLLVAL_FINE(fine);

  /* Wait for the write to finish. */
  while (!SYSCTRL->PCLKSR.bit.DFLLRDY) {};

  SYSCTRL->DFLLCTRL.reg |=
      /* Enable USB clock recovery mode */
      SYSCTRL_DFLLCTRL_USBCRM |
      /* Disable chill cycle as per datasheet to speed up locking.
        This is specified in section 17.6.7.2.2, and chill cycles
        are described in section 17.6.7.2.1. */
      SYSCTRL_DFLLCTRL_CCDIS;

  /* Configure the DFLL to multiply the 1 kHz clock to 48 MHz */
  SYSCTRL->DFLLMUL.reg =
      /* This value is output frequency / reference clock frequency,
        so 48 MHz / 1 kHz */
      SYSCTRL_DFLLMUL_MUL(48000) |
      /* The coarse and fine values can be set to their minimum
        since coarse is fixed in USB clock recovery mode and
        fine should lock on quickly. */
      SYSCTRL_DFLLMUL_FSTEP(1) |
      SYSCTRL_DFLLMUL_CSTEP(1);

  /* Closed loop mode */
  SYSCTRL->DFLLCTRL.bit.MODE = 1;

  /* Enable the DFLL */
  SYSCTRL->DFLLCTRL.bit.ENABLE = 1;

  /* Wait for the write to complete */
  while (!SYSCTRL->PCLKSR.bit.DFLLRDY) {};      

  /* Setup GCLK0 using the DFLL @ 48 MHz */
  GCLK->GENCTRL.reg =
      GCLK_GENCTRL_ID(0) |
      GCLK_GENCTRL_SRC_DFLL48M |
      /* Improve the duty cycle. */
      GCLK_GENCTRL_IDC |
      GCLK_GENCTRL_GENEN;

  /* Wait for the write to complete */
  while(GCLK->STATUS.bit.SYNCBUSY) {};
}

void setup8MhzGCLK1(){
  // from https://blog.thea.codes/understanding-the-sam-d21-clocks/
  /* Configure GCLK1's divider - in this case, no division - so just divide by one */
  GCLK->GENDIV.reg =
      GCLK_GENDIV_ID(1) |
      GCLK_GENDIV_DIV(1);

  /* Setup GCLK1 using the internal 8 MHz oscillator */
  GCLK->GENCTRL.reg =
      GCLK_GENCTRL_ID(1) |
      GCLK_GENCTRL_SRC_OSC8M |
      /* Improve the duty cycle. */
      GCLK_GENCTRL_IDC |
      GCLK_GENCTRL_GENEN;

  /* Wait for the write to complete */
  while(GCLK->STATUS.bit.SYNCBUSY) {};

}

void setupADC(){
  setup8MhzGCLK1();

  // from https://blog.thea.codes/reading-analog-values-with-the-samd-adc/
  /* Connect GCLK1 ADC. */
  PM->APBCMASK.reg |= PM_APBCMASK_ADC;
  /* Enable GCLK1 for the ADC */
  GCLK->CLKCTRL.reg = GCLK_CLKCTRL_CLKEN |
                      GCLK_CLKCTRL_GEN_GCLK1 |
                      GCLK_CLKCTRL_ID_ADC;

  /* Wait for bus synchronization. */
  while (GCLK->STATUS.bit.SYNCBUSY) {};

  /* Enable GCLK1 for the ADC */
  GCLK->CLKCTRL.reg = GCLK_CLKCTRL_CLKEN |
                      GCLK_CLKCTRL_GEN_GCLK1 |
                      GCLK_CLKCTRL_ID_ADC;

  /* Wait for bus synchronization. */
  while (GCLK->STATUS.bit.SYNCBUSY) {};

  /* load factory calibration from chip's NVM */
  uint32_t bias = (*((uint32_t *) ADC_FUSES_BIASCAL_ADDR) & ADC_FUSES_BIASCAL_Msk) >> ADC_FUSES_BIASCAL_Pos;
  uint32_t linearity = (*((uint32_t *) ADC_FUSES_LINEARITY_0_ADDR) & ADC_FUSES_LINEARITY_0_Msk) >> ADC_FUSES_LINEARITY_0_Pos;
  linearity |= ((*((uint32_t *) ADC_FUSES_LINEARITY_1_ADDR) & ADC_FUSES_LINEARITY_1_Msk) >> ADC_FUSES_LINEARITY_1_Pos) << 5;

  /* Wait for bus synchronization. */
  while (ADC->STATUS.bit.SYNCBUSY) {};

  /* Write the calibration data. */
  ADC->CALIB.reg = ADC_CALIB_BIAS_CAL(bias) | ADC_CALIB_LINEARITY_CAL(linearity);

  // set up measurement parameters
  /* Wait for bus synchronization. */
  while (ADC->STATUS.bit.SYNCBUSY) {};

  /* Use the internal VCC reference. This is 1/2 of what's on VCCA.
    since VCCA is typically 3.3v, this is 1.65v.
  */
  ADC->REFCTRL.reg = ADC_REFCTRL_REFSEL_INTVCC1;

  /* Only capture one sample. The ADC can actually capture and average multiple
    samples for better accuracy, but there's no need to do that for this
    example.
  */
  ADC->AVGCTRL.reg = ADC_AVGCTRL_SAMPLENUM_1;

  /* Set the clock prescaler to 4, which will run the ADC at
    8 Mhz / 4 = 2MHz. Max ADC clock frequency according to datasheet is 2.1Mhz (2100 kHz, datasheet 37.11.4 table 37-24)
    Set the resolution to 12bit.
  */
  ADC->CTRLB.reg = ADC_CTRLB_PRESCALER_DIV4 |
                  ADC_CTRLB_RESSEL_12BIT;

  // The I/O pins are assembled in pin groups (”PORT groups”) with up to 32 pins. 
  // Group 0 consists of the PA pins, group 1 is for the PB pins, etc. 
  // Each pin group has its own PORT registers, with a 0x80 address spacing. 
  // For example, the register address offset for the Data Direction (DIR) register for group 0  
  // (PA00 to PA31) is 0x00, and the register address offset for the DIR register for group 1 (PB00 to PB31) is
  // 0x80.

  /* Set input pins. */
  PORT->Group[0].DIRCLR.reg = PORT_PA02 |
                              PORT_PA03 |
                              PORT_PA04 |
                              PORT_PA05 |
                              PORT_PA06 |
                              PORT_PA07 |
                              PORT_PA08 |
                              PORT_PA09 |
                              PORT_PA10 |
                              PORT_PA11 ;
  PORT->Group[1].DIRCLR.reg = PORT_PB00 |
                              PORT_PB01 |
                              PORT_PB02 |
                              PORT_PB03 |
                              PORT_PB04 |
                              PORT_PB05 |
                              PORT_PB06 |
                              PORT_PB07 |
                              PORT_PB08 |
                              PORT_PB09 ;

  PORT->Group[0].PINCFG[2].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PA02.
  PORT->Group[0].PINCFG[3].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PA03.
  PORT->Group[0].PINCFG[4].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PA04.
  PORT->Group[0].PINCFG[5].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PA05.
  PORT->Group[0].PINCFG[6].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PA06.
  PORT->Group[0].PINCFG[7].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PA07.
  PORT->Group[0].PINCFG[8].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PA08.
  PORT->Group[0].PINCFG[9].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PA09.
  PORT->Group[0].PINCFG[10].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PA10.
  PORT->Group[0].PINCFG[11].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PA11.

  PORT->Group[1].PINCFG[0].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PB00.
  PORT->Group[1].PINCFG[1].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PB01.
  PORT->Group[1].PINCFG[2].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PB02.
  PORT->Group[1].PINCFG[3].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PB03.
  PORT->Group[1].PINCFG[4].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PB04.
  PORT->Group[1].PINCFG[5].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PB05.
  PORT->Group[1].PINCFG[6].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PB06.
  PORT->Group[1].PINCFG[7].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PB07.
  PORT->Group[1].PINCFG[8].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PB08.
  PORT->Group[1].PINCFG[9].reg |= PORT_PINCFG_PMUXEN; // Enable the peripheral multiplexer for PB09.

  /* each PMUX register controls a pair of pins. 
  The even numbered pin is controlled by the PMUXE (even) bitfield and the odd numbered pin is controlled by the PMUXO (odd) bitfield.
  e.g. PB06 corresponds to Group1 (Port B) and is an even numbered pin, so it is controlled by the PMUXE bitfield.
  The PMUX register index for PB06 would be half of the pin number (since each PMUX controls a pair of pins), which is 3 (6/2).
  So, to set PB06 to analog input (function B), you would write:
  PORT->Group[1].PMUX[3].bit.reg = PORT_PMUX_PMUXE_B;
*/

  PORT->Group[0].PMUX[1].reg = PORT_PMUX_PMUXE_B; // Set PA02 to function B which is analog input.
  PORT->Group[0].PMUX[1].reg = PORT_PMUX_PMUXO_B; // Set PA03 to function B which is analog input.
  PORT->Group[0].PMUX[2].reg = PORT_PMUX_PMUXE_B; // Set PA04 to function B which is analog input.
  PORT->Group[0].PMUX[2].reg = PORT_PMUX_PMUXO_B; // Set PA05 to function B which is analog input.
  PORT->Group[0].PMUX[3].reg = PORT_PMUX_PMUXE_B; // Set PA06 to function B which is analog input.
  PORT->Group[0].PMUX[3].reg = PORT_PMUX_PMUXO_B; // Set PA07 to function B which is analog input.
  PORT->Group[0].PMUX[4].reg = PORT_PMUX_PMUXE_B; // Set PA08 to function B which is analog input.
  PORT->Group[0].PMUX[4].reg = PORT_PMUX_PMUXO_B; // Set PA09 to function B which is analog input.
  PORT->Group[0].PMUX[5].reg = PORT_PMUX_PMUXE_B; // Set PA10 to function B which is analog input.
  PORT->Group[0].PMUX[5].reg = PORT_PMUX_PMUXO_B; // Set PA11 to function B which is analog input.

  PORT->Group[1].PMUX[0].reg = PORT_PMUX_PMUXE_B; // Set PB00 to function B which is analog input.
  PORT->Group[1].PMUX[0].reg = PORT_PMUX_PMUXO_B; // Set PB01 to function B which is analog input.
  PORT->Group[1].PMUX[1].reg = PORT_PMUX_PMUXE_B; // Set PB02 to function B which is analog input.
  PORT->Group[1].PMUX[1].reg = PORT_PMUX_PMUXO_B; // Set PB03 to function B which is analog input.
  PORT->Group[1].PMUX[2].reg = PORT_PMUX_PMUXE_B; // Set PB04 to function B which is analog input.
  PORT->Group[1].PMUX[2].reg = PORT_PMUX_PMUXO_B; // Set PB05 to function B which is analog input.
  PORT->Group[1].PMUX[3].reg = PORT_PMUX_PMUXE_B; // Set PB06 to function B which is analog input.
  PORT->Group[1].PMUX[3].reg = PORT_PMUX_PMUXO_B; // Set PB07 to function B which is analog input.
  PORT->Group[1].PMUX[4].reg = PORT_PMUX_PMUXE_B; // Set PB08 to function B which is analog input.
  PORT->Group[1].PMUX[4].reg = PORT_PMUX_PMUXO_B; // Set PB09 to function B which is analog input.

  /* Configure the input parameters.

    - GAIN_DIV2 means that the input voltage is halved. This is important
      because the voltage reference is 1/2 of VCCA. So if you want to
      measure 0-3.3v, you need to halve the input as well.

    - MUXNEG_GND means that the ADC should compare the input value to GND.
  */
  // ADC->INPUTCTRL.reg = ADC_INPUTCTRL_GAIN_DIV2 |
  //                     ADC_INPUTCTRL_MUXNEG_GND;
  /* Wait for bus synchronization. */
  while (ADC->STATUS.bit.SYNCBUSY) {};
}

void setADCToPin(unsigned long sensor){ // read pin 
  // from https://blog.thea.codes/reading-analog-values-with-the-samd-adc/

  /* Configure the input parameters.

    - GAIN_DIV2 means that the input voltage is halved. This is important
      because the voltage reference is 1/2 of VCCA. So if you want to
      measure 0-3.3v, you need to halve the input as well.

    - MUXNEG_GND means that the ADC should compare the input value to GND.

    - MUXPOS_PIN14 means that the ADC should read from AIN14, or PB06. Positive Mux Input Selection. These bits define the Mux selection for the positive ADC input.

    MUXPOS_PIN0  = AIN0  = PA02 -> SENSOR4
    MUXPOS_PIN1  = AIN1  = PA03 -> SENSOR5
    MUXPOS_PIN2  = AIN2  = PB08 -> SENSOR19
    MUXPOS_PIN3  = AIN3  = PB09 -> SENSOR14
    MUXPOS_PIN4  = AIN4  = PA04 -> SENSOR18
    MUXPOS_PIN5  = AIN5  = PA05 -> SENSOR13
    MUXPOS_PIN6  = AIN6  = PA06 -> SENSOR17
    MUXPOS_PIN7  = AIN7  = PA07 -> SENSOR12
    MUXPOS_PIN8  = AIN8  = PB00 -> SENSOR8
    MUXPOS_PIN9  = AIN9  = PB01 -> SENSOR9
    MUXPOS_PIN10 = AIN10 = PB02 -> SENSOR6
    MUXPOS_PIN11 = AIN11 = PB03 -> SENSOR7
    MUXPOS_PIN12 = AIN12 = PB04 -> SENSOR2
    MUXPOS_PIN13 = AIN13 = PB05 -> SENSOR3
    MUXPOS_PIN14 = AIN14 = PB06 -> SENSOR0
    MUXPOS_PIN15 = AIN15 = PB07 -> SENSOR1
    MUXPOS_PIN16 = AIN16 = PA08 -> SENSOR16
    MUXPOS_PIN17 = AIN17 = PA09 -> SENSOR11
    MUXPOS_PIN18 = AIN18 = PA10 -> SENSOR15
    MUXPOS_PIN19 = AIN19 = PA11 -> SENSOR10
    
  */

  ADC->INPUTCTRL.reg = ADC_INPUTCTRL_GAIN_DIV2 |
                      ADC_INPUTCTRL_MUXNEG_GND | sensor;

  /* Wait for bus synchronization. */
  while (ADC->STATUS.bit.SYNCBUSY) {};

  /* Enable the ADC. */
  ADC->CTRLA.bit.ENABLE = true;  
}

uint32_t readADC(bool disregardFirstMeasurement) {
  /* Wait for bus synchronization. */
  while (ADC->STATUS.bit.SYNCBUSY) {};

  /* Start the ADC using a software trigger. */
  ADC->SWTRIG.bit.START = true;

  /* Wait for the result ready flag to be set. */
  while (ADC->INTFLAG.bit.RESRDY == 0);

  /* Clear the flag. */
  ADC->INTFLAG.reg = ADC_INTFLAG_RESRDY;

  if (disregardFirstMeasurement) {
    // have to do readings twice since the first conversion after the reference is changed must not be used. 
    // see https://github.com/arduino/ArduinoCore-samd/blob/master/cores/arduino/wiring_analog.c#L128
    // the first reading is not accurate at all as per note in the datasheet. E.g. after changing ADC channels.

    /* Wait for bus synchronization. */
    while (ADC->STATUS.bit.SYNCBUSY) {};

    /* Start the ADC using a software trigger. */
    ADC->SWTRIG.bit.START = true;

    /* Wait for the result ready flag to be set. */
    while (ADC->INTFLAG.bit.RESRDY == 0);

    /* Clear the flag. */
    ADC->INTFLAG.reg = ADC_INTFLAG_RESRDY;
  }

  /* Read the value. */
  uint32_t result = ADC->RESULT.reg;  
  return result;
}
