
#define CPU_HZ 48000000 // The cpu clock is running at 48 MHz
#define TICK_HZ_1MS 1000 // We want a 1 ms tick.
#define COUNTS_PER_TICK_1MS (CPU_HZ / TICK_HZ_1MS) // This is the number of clock cycles per tick
#define TICK_HZ_1US 1000000 // We want a 1 us tick.
#define COUNTS_PER_TICK_1US (CPU_HZ / TICK_HZ_1US) // This is the number of clock cycles per tick
#define TICK_HZ_ADC 20000 // We want a 200 us tick for the ADC.
#define COUNTS_PER_TICK_ADC (CPU_HZ / TICK_HZ_ADC) // This is the number of clock cycles per tick



#define TESTPIN PORT_PB10 // temp test pin
void TC3_Handler() { // 1ms timer
   PORT->Group[1].OUTTGL.reg = TESTPIN; // toggle TEST pin to be able to check with oscilloscope
  // Clear the interrupt
  TC3->COUNT16.INTFLAG.bit.MC0 = 1;
}
void TC4_Handler() { // 1us timer
  // PORT->Group[1].OUTTGL.reg = TESTPIN; // toggle TEST pin to be able to check with oscilloscope
  // Clear the interrupt
  TC4->COUNT16.INTFLAG.bit.MC0 = 1;
}
void TC5_Handler() { // 200 us timer
  // PORT->Group[1].OUTTGL.reg = TESTPIN; // toggle TEST pin to be able to check with oscilloscope
  // Clear the interrupt
  TC5->COUNT16.INTFLAG.bit.MC0 = 1;
}


void setup() {
    PORT->Group[1].DIRSET.reg |= TESTPIN; // set test pin as output
    // Set up the generic clock (GCLK4) used to clock timers
    GCLK->GENDIV.reg = GCLK_GENDIV_DIV(1) |          // Divide the 48MHz clock source by divisor 1: 48MHz/1=48MHz
                       GCLK_GENDIV_ID(4);            // Select Generic Clock (GCLK) 4
    while (GCLK->STATUS.bit.SYNCBUSY);               // Wait for synchronization

    GCLK->GENCTRL.reg = GCLK_GENCTRL_IDC |           // Set the duty cycle to 50/50 HIGH/LOW
                        GCLK_GENCTRL_GENEN |         // Enable GCLK4
                        GCLK_GENCTRL_SRC_DFLL48M |   // Set the 48MHz clock source
                        GCLK_GENCTRL_ID(4);          // Select GCLK4
    while (GCLK->STATUS.bit.SYNCBUSY);               // Wait for synchronization

    // Feed GCLK4 to TC3
    GCLK->CLKCTRL.reg = GCLK_CLKCTRL_CLKEN |         // Enable GCLK4
                        GCLK_CLKCTRL_GEN_GCLK4 |     // Select GCLK4
                        GCLK_CLKCTRL_ID_TCC2_TC3;     // Feed the GCLK4 to TC3 (and TCC2)
    while (GCLK->STATUS.bit.SYNCBUSY);               // Wait for synchronization

    // Feed GCLK4 to TC4 and TC5
    GCLK->CLKCTRL.reg = GCLK_CLKCTRL_CLKEN |         // Enable GCLK4
                        GCLK_CLKCTRL_GEN_GCLK4 |     // Select GCLK4
                        GCLK_CLKCTRL_ID_TC4_TC5;     // Feed the GCLK4 to TC4 and TC5
    while (GCLK->STATUS.bit.SYNCBUSY);               // Wait for synchronization

    // specific for TC3: 1ms
    TC3->COUNT16.CTRLA.reg |= TC_CTRLA_MODE_COUNT16;  // Set Timer counter Mode to 16 bits
    while (TC3->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    TC3->COUNT16.CTRLA.reg |= TC_CTRLA_WAVEGEN_MFRQ;  // Set TC as Match Frequency
    while (TC3->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    TC3->COUNT16.CC[0].reg = COUNTS_PER_TICK_1MS;     // Set the TC3 CC0 register as the TOP value in match frequency mode
    while (TC3->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable the TC3 interrupt request
    TC3->COUNT16.INTENSET.bit.MC0 = 1;
    while (TC3->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable TC3
    TC3->COUNT16.CTRLA.reg |= TC_CTRLA_ENABLE;        // Enable TC3
    while (TC3->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable the TC3 interrupt vector use priority 2
    NVIC_SetPriority(TC3_IRQn, 2);                    // Set the NVIC priority for TC3 to 2
    NVIC_EnableIRQ(TC3_IRQn);

    // specific for TC4: 1us
    TC4->COUNT16.CTRLA.reg |= TC_CTRLA_MODE_COUNT16;  // Set Timer counter Mode to 16 bits
    while (TC4->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    TC4->COUNT16.CTRLA.reg |= TC_CTRLA_WAVEGEN_MFRQ;  // Set TC as Match Frequency
    while (TC4->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    TC4->COUNT16.CC[0].reg = COUNTS_PER_TICK_1US;     // Set the TC4 CC0 register as the TOP value in match frequency mode
    while (TC4->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable the TC4 interrupt request
    TC4->COUNT16.INTENSET.bit.MC0 = 1;
    while (TC4->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable TC4
    TC4->COUNT16.CTRLA.reg |= TC_CTRLA_ENABLE;        // Enable TC3
    while (TC4->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable the TC4 interrupt vector use priority 0
    NVIC_SetPriority(TC4_IRQn, 0);                    // Set the NVIC priority for TC4 to 0
    NVIC_EnableIRQ(TC4_IRQn);

    // specific for TC5: 200us
    TC5->COUNT16.CTRLA.reg |= TC_CTRLA_MODE_COUNT16;  // Set Timer counter Mode to 16 bits
    while (TC5->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    TC5->COUNT16.CTRLA.reg |= TC_CTRLA_WAVEGEN_MFRQ;  // Set TC as Match Frequency
    while (TC5->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    TC5->COUNT16.CC[0].reg = COUNTS_PER_TICK_ADC;     // Set the TC5 CC0 register as the TOP value in match frequency mode
    while (TC5->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable the TC5 interrupt request
    TC5->COUNT16.INTENSET.bit.MC0 = 1;
    while (TC5->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable TC5
    TC5->COUNT16.CTRLA.reg |= TC_CTRLA_ENABLE;        // Enable TC5
    while (TC5->COUNT16.STATUS.bit.SYNCBUSY);         // Wait for synchronization

    // Enable the TC5 interrupt vector use priority 1
    NVIC_SetPriority(TC5_IRQn, 1);                    // Set the NVIC priority for TC5 to 1
    NVIC_EnableIRQ(TC5_IRQn);
}

void loop() {
    // Your main code here...
}
